# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest
from unittest.mock import MagicMock

from tests.plugins import TemporaryDirectory
from troubadix.standalone_plugins.last_modification import parse_args, update


class ParseArgsTestCase(unittest.TestCase):
    def test_parse_files(self):
        with TemporaryDirectory() as tempdir:
            testfile1 = tempdir / "testfile1.nasl"
            testfile2 = tempdir / "testfile2.nasl"

            testfile1.touch()
            testfile2.touch()

            args = parse_args(["--file", str(testfile1), str(testfile2)])

            self.assertEqual(args.files[0], testfile1)
            self.assertEqual(args.files[1], testfile2)

    def test_parse_from_file(self):
        with TemporaryDirectory() as tempdir:
            from_file = tempdir / "from_file.txt"
            testfile1 = tempdir / "testfile1.nasl"
            testfile2 = tempdir / "testfile2.nasl"

            from_file.write_text(f"{testfile1}\n{testfile2}\n", encoding="utf8")

            args = parse_args(["--from-file", str(from_file)])

            self.assertEqual(args.from_file, from_file)


class UpdateTestCase(unittest.TestCase):
    def test_update(self):
        terminal = MagicMock()
        with TemporaryDirectory() as tempdir:
            content = (
                'script_version("2021-07-19T12:32:02+0000");\n'
                'script_tag(name: "last_modification", value: "2021-07-19 '
                '12:32:02 +0000 (Mon, 19 Jul 2021)");\n'
            )
            testfile1 = tempdir / "testfile1.nasl"
            testfile1.write_text(content, encoding="utf8")

            update(testfile1, terminal)

            new_content = testfile1.read_text(encoding="utf8")

            self.assertNotEqual(content, new_content)

    def test_update_invalid_date(self):
        terminal = MagicMock()
        with TemporaryDirectory() as tempdir:
            content = (
                'script_version("foo");\n'
                'script_tag(name: "last_modification", value: "bar");\n'
            )
            testfile1 = tempdir / "testfile1.nasl"
            testfile1.write_text(content, encoding="utf8")

            update(testfile1, terminal)

            new_content = testfile1.read_text(encoding="utf8")

            self.assertNotEqual(content, new_content)

    def test_no_update_missing_script_version(self):
        terminal = MagicMock()
        with TemporaryDirectory() as tempdir:
            content = 'script_tag(name: "last_modification", value: "bar");\n'
            testfile1 = tempdir / "testfile1.nasl"
            testfile1.write_text(content, encoding="utf8")

            update(testfile1, terminal)

            new_content = testfile1.read_text(encoding="utf8")

            self.assertEqual(content, new_content)

    def test_no_update_missing_last_modification_tag(self):
        terminal = MagicMock()
        with TemporaryDirectory() as tempdir:
            content = 'script_version("2021-07-19T12:32:02+0000");\n'
            testfile1 = tempdir / "testfile1.nasl"
            testfile1.write_text(content, encoding="utf8")

            update(testfile1, terminal)

            new_content = testfile1.read_text(encoding="utf8")

            self.assertEqual(content, new_content)



# Copyright (C) 2021 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

""" checking badwords in NASL scripts with the NASLinter """

from pathlib import Path
from typing import Iterable, Iterator

from troubadix.helper import is_ignore_file
from troubadix.plugin import LineContentPlugin, LinterError, LinterResult

# hexstr(OpenVAS) = '4f70656e564153'
# hexstr(openvas) = '6f70656e766173'
DEFAULT_BADWORDS = [
    "cracker",
    "openvas",
    "OpenVAS",
    "4f70656e564153",
    "6f70656e766173",
]

_IGNORE_FILES = [
    "gb_openvas",
    "gb_gsa_",
    "http_func.inc",
    "misc_func.inc",
    "OpenVAS_detect.nasl",
]

EXCEPTIONS = [
    "openvas-nasl",
    "openvas-smb",
    "openvas-scanner",
    "openvas-libraries",
    "openvas-gsa",
    "openvas-cli",
    "openvas-manager",
    "openvassd",
    "openvasd",
    "lists.wald.intevation.org",
    "lib64openvas-devel",
    "lib64openvas6",
    "libopenvas-devel",
    "libopenvas6",
    "cpe:/a:openvas",
    "OPENVAS_VERSION",
    "openvas.org",
    "get_preference",
    "OPENVAS_USE_LIBSSH",
    "github.com/greenbone",
    "Cookie: mstshash=openvas",
    "smb_nt.inc",
    "lanman",
    "OpenVAS_detect.nasl",
    "OpenVAS TCP Scanner",
    "openvas_tcp_scanner",
    "gb_openvas_",
    "OpenVAS_detect.nasl",
    "OpenVAS Manager",
    "OpenVAS Administrator",
    "OpenVAS / Greenbone Vulnerability Manager",
    "openvas_1808149858",
    "evil.zip -> openvas.jsp",
    'url = "/openvas.jsp";',
    'if( "OpenVAS RCE Test" >< buf )',
    'the file "/openvas.jsp" was created',
]

STARTS_WITH_EXCEPTIONS = [
    "# OpenVAS Vulnerability Test",
    "# OpenVAS Include File",
    "  script_",
    "# $Id: ",
]

COMBINED = [("find_service3.nasl", "OpenVAS-")]


class CheckBadwords(LineContentPlugin):
    """This plugin checks the passed VT for the use of any of
    the defined badwords. An error will be thrown if the VT contains
    such a badword.
    """

    name = "check_badwords"

    def check_lines(
        self,
        nasl_file: Path,
        lines: Iterable[str],
    ) -> Iterator[LinterResult]:
        if is_ignore_file(nasl_file, _IGNORE_FILES):
            return

        for i, line in enumerate(lines, 1):
            if any(badword in line for badword in DEFAULT_BADWORDS):
                if (
                    not any(exception in line for exception in EXCEPTIONS)
                    and not any(
                        line.startswith(start)
                        for start in STARTS_WITH_EXCEPTIONS
                    )
                    and not any(
                        nasl_file.name == filename and value in line
                        for filename, value in COMBINED
                    )
                ):
                    yield LinterError(
                        f"Badword in line {i:5}: {line}",
                        plugin=self.name,
                        file=nasl_file,
                        line=i,
                    )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# pylint: disable=protected-access

import io
import unittest
from contextlib import redirect_stdout
from pathlib import Path

from pontos.terminal.terminal import ConsoleTerminal

from troubadix.helper import CURRENT_ENCODING
from troubadix.helper.helper import get_path_from_root
from troubadix.plugins import _FILE_PLUGINS, _FILES_PLUGINS
from troubadix.plugins.badwords import CheckBadwords
from troubadix.plugins.copyright_text import CheckCopyrightText
from troubadix.plugins.cvss_format import CheckCVSSFormat
from troubadix.plugins.duplicate_oid import CheckDuplicateOID
from troubadix.plugins.missing_desc_exit import CheckMissingDescExit
from troubadix.plugins.no_solution import CheckNoSolution
from troubadix.plugins.script_version_and_last_modification_tags import (
    CheckScriptVersionAndLastModificationTags,
)
from troubadix.reporter import Reporter
from troubadix.runner import Runner, TroubadixException

_here = Path(__file__).parent


class TestRunner(unittest.TestCase):
    def setUp(self):
        self._term = ConsoleTerminal()
        self.root = _here / "plugins" / "test_files" / "nasl"
        self._reporter = Reporter(term=self._term, root=self.root)

    def test_runner_with_all_plugins(self):
        runner = Runner(
            n_jobs=1,
            reporter=self._reporter,
            root=self.root,
        )

        plugins = _FILE_PLUGINS + _FILES_PLUGINS

        for plugin in runner.plugins:
            self.assertIn(plugin, plugins)

    def test_runner_with_excluded_plugins(self):
        excluded_plugins = [
            "CheckBadwords",
            "CheckCopyRightYearPlugin",
        ]
        included_plugins = [
            plugin.__name__
            for plugin in _FILE_PLUGINS + _FILES_PLUGINS
            if plugin.__name__ not in excluded_plugins
        ]
        runner = Runner(
            n_jobs=1,
            reporter=self._reporter,
            excluded_plugins=excluded_plugins,
            root=self.root,
        )

        for plugin in runner.plugins:
            self.assertIn(plugin.__name__, included_plugins)

    def test_runner_with_included_plugins(self):
        included_plugins = [
            CheckBadwords.name,
            CheckCopyrightText.name,
        ]
        runner = Runner(
            n_jobs=1,
            reporter=self._reporter,
            included_plugins=included_plugins,
            root=self.root,
        )

        self.assertEqual(len(runner.plugins), 2)

        for plugin in runner.plugins:
            self.assertIn(plugin.name, included_plugins)

    def test_runner_run_ok(self):
        nasl_file = (
            _here
            / "plugins"
            / "test_files"
            / "nasl"
            / "21.04"
            / "runner"
            / "test_valid_oid.nasl"
        )
        included_plugins = [
            CheckMissingDescExit.name,
        ]
        runner = Runner(
            n_jobs=1,
            reporter=self._reporter,
            included_plugins=included_plugins,
            root=self.root,
        )
        with redirect_stdout(io.StringIO()) as _:
            sys_exit = runner.run([nasl_file])

        self.assertTrue(sys_exit)
        self.assertEqual(self._reporter._result_counts.error_count, 0)
        self.assertEqual(self._reporter._result_counts.warning_count, 0)
        self.assertEqual(self._reporter._result_counts.fix_count, 0)
        self.assertEqual(
            self._reporter._result_counts.result_counts[
                CheckMissingDescExit.name
            ]["error"],
            0,
        )
        self.assertEqual(
            self._reporter._result_counts.result_counts[
                CheckMissingDescExit.name
            ]["warning"],
            0,
        )
        self.assertEqual(
            self._reporter._result_counts.result_counts[
                CheckMissingDescExit.name
            ]["fix"],
            0,
        )

    def test_runner_run_error(self):
        nasl_file = (
            _here
            / "plugins"
            / "test_files"
            / "nasl"
            / "21.04"
            / "runner"
            / "fail.nasl"
        )
        included_plugins = [CheckCVSSFormat.name, CheckNoSolution.name]
        runner = Runner(
            n_jobs=1,
            reporter=self._reporter,
            included_plugins=included_plugins,
            root=self.root,
        )

        with redirect_stdout(io.StringIO()) as _:
            sys_exit = runner.run([nasl_file])

        self.assertFalse(sys_exit)

        self.assertEqual(self._reporter._result_counts.error_count, 3)
        self.assertEqual(self._reporter._result_counts.warning_count, 0)
        self.assertEqual(self._reporter._result_counts.fix_count, 0)
        self.assertEqual(
            self._reporter._result_counts.result_counts[CheckCVSSFormat.name][
                "error"
            ],
            2,
        )
        self.assertEqual(
            self._reporter._result_counts.result_counts[CheckCVSSFormat.name][
                "warning"
            ],
            0,
        )
        self.assertEqual(
            self._reporter._result_counts.result_counts[CheckCVSSFormat.name][
                "fix"
            ],
            0,
        )

    def test_runner_run_fail_with_verbose_level_2(self):
        nasl_file = (
            _here
            / "plugins"
            / "test_files"
            / "nasl"
            / "21.04"
            / "runner"
            / "fail.nasl"
        )
        content = nasl_file.read_text(encoding=CURRENT_ENCODING)

        reporter = Reporter(term=self._term, root=self.root, verbose=2)

        runner = Runner(
            n_jobs=1,
            reporter=reporter,
            included_plugins=[CheckScriptVersionAndLastModificationTags.name],
            root=self.root,
        )

        with redirect_stdout(io.StringIO()) as f:
            runner.run([nasl_file])

            new_content = nasl_file.read_text(encoding=CURRENT_ENCODING)
            self.assertEqual(content, new_content)

        output = f.getvalue()
        self.assertIn(
            f"Checking {get_path_from_root(nasl_file, self.root)}",
            output,
        )
        self.assertIn(
            "Results for plugin "
            "check_script_version_and_last_modification_tags",
            output,
        )
        # CI terminal formats for 80 chars per line
        self.assertIn(
            "VT is missing script_version();.",
            output,
        )

    def test_runner_run_ok_with_verbose_level_3(self):
        included_plugins = [
            "CheckMissingDescExit",
        ]
        nasl_file = (
            _here
            / "plugins"
            / "test_files"
            / "nasl"
            / "21.04"
            / "runner"
            / "test.nasl"
        )
        content = nasl_file.read_text(encoding=CURRENT_ENCODING)

        reporter = Reporter(term=self._term, root=self.root, verbose=3)

        runner = Runner(
            n_jobs=1,
            reporter=reporter,
            included_plugins=included_plugins,
            root=self.root,
        )

        with redirect_stdout(io.StringIO()) as f:
            runner.run([nasl_file])

            new_content = nasl_file.read_text(encoding=CURRENT_ENCODING)
            self.assertEqual(content, new_content)

        output = f.getvalue()
        self.assertIn(
            f"Checking {get_path_from_root(nasl_file, self.root)}", output
        )
        self.assertIn("No results for plugin check_missing_desc_exit", output)

    def test_runner_run_ok_with_verbose_level_2(self):
        included_plugins = [
            "CheckMissingDescExit",
        ]
        nasl_file = (
            _here
            / "plugins"
            / "test_files"
            / "nasl"
            / "21.04"
            / "runner"
            / "test.nasl"
        )
        content = nasl_file.read_text(encoding=CURRENT_ENCODING)

        reporter = Reporter(term=self._term, root=self.root, verbose=2)

        runner = Runner(
            n_jobs=1,
            reporter=reporter,
            included_plugins=included_plugins,
            root=self.root,
        )

        with redirect_stdout(io.StringIO()) as f:
            runner.run([nasl_file])

            new_content = nasl_file.read_text(encoding=CURRENT_ENCODING)
            self.assertEqual(content, new_content)

        output = f.getvalue()
        self.assertIn(
            "Checking " f"{get_path_from_root(nasl_file, self.root)}",
            output,
        )
        self.assertNotIn(
            "No results for plugin check_missing_desc_exit", output
        )

    def test_runner_run_ok_with_verbose_level_1(self):
        included_plugins = [
            "CheckMissingDescExit",
        ]
        nasl_file = (
            _here
            / "plugins"
            / "test_files"
            / "nasl"
            / "21.04"
            / "runner"
            / "test.nasl"
        )
        content = nasl_file.read_text(encoding=CURRENT_ENCODING)

        reporter = Reporter(term=self._term, root=self.root, verbose=1)

        runner = Runner(
            reporter=reporter,
            n_jobs=1,
            included_plugins=included_plugins,
            root=self.root,
        )

        with redirect_stdout(io.StringIO()) as f:
            runner.run([nasl_file])

            new_content = nasl_file.read_text(encoding=CURRENT_ENCODING)
            self.assertEqual(content, new_content)

        output = f.getvalue()
        self.assertNotIn(
            "No results for plugin check_missing_desc_exit", output
        )
        self.assertNotIn("Results for plugin check_missing_desc_exit", output)

        # revert changes for the next time
        nasl_file.write_text(content, encoding=CURRENT_ENCODING)

    def test_no_plugins(self):
        runner = Runner(
            n_jobs=1,
            reporter=self._reporter,
            included_plugins=["foo"],
            root=self.root,
        )

        nasl_file = _here / "plugins" / "test.nasl"

        with self.assertRaises(TroubadixException):
            runner.run([nasl_file])

    def test_runner_log_file(self):
        included_plugins = [
            CheckDuplicateOID.name,
            CheckMissingDescExit.name,
            CheckNoSolution.name,
        ]
        nasl_file = (
            _here
            / "plugins"
            / "test_files"
            / "nasl"
            / "21.04"
            / "runner"
            / "test.nasl"
        )
        gen_log_file = _here / "gen_log.txt"

        reporter = Reporter(
            term=self._term, root=self.root, verbose=3, log_file=gen_log_file
        )

        runner = Runner(
            reporter=reporter,
            n_jobs=1,
            included_plugins=included_plugins,
            root=self.root,
        )
        with redirect_stdout(io.StringIO()):
            runner.run([nasl_file])

        compare_content = (
            "\tIncluded Plugins: check_duplicate_oid, check_missing_desc_exit"
            ", check_no_solution\n"
            "\tRunning plugins: check_duplicate_oid, check_no_solution, "
            "check_missing_desc_exit\n"
            "\n\nRun plugin check_duplicate_oid\n"
            "\tResults for plugin check_duplicate_oid\n"
            "\t\tInvalid OID 1.2.3.4.5.6.78909.1.7.654321 found"
            " in '21.04/runner/test.nasl'.\n\n\n"
            "Run plugin check_no_solution\n"
            "\t\tNo results for plugin check_no_solution\n\n\n"
            f"Checking {get_path_from_root(nasl_file, self.root)} (1/1)\n\t\t"
            "No results for plugin"
            " check_missing_desc_exit\n\tTime elapsed: 0:00:00.013967"
        )
        gen_content = gen_log_file.read_text(encoding="utf-8")
        gen_log_file.unlink()
        # Remove Time elapsed line
        self.assertEqual(
            compare_content.splitlines()[:-1],
            gen_content.splitlines()[:-1],
        )

    def test_runner_log_file_fail(self):
        included_plugins = [
            "CheckMissingDescExit",
        ]
        nasl_file = (
            _here
            / "plugins"
            / "test_files"
            / "nasl"
            / "21.04"
            / "runner"
            / "test.nasl"
        )
        gen_log_file = _here / "gen_log.txt"

        reporter = Reporter(
            term=self._term, root=self.root, log_file=gen_log_file
        )

        runner = Runner(
            n_jobs=1,
            reporter=reporter,
            included_plugins=included_plugins,
            root=self.root,
        )
        with redirect_stdout(io.StringIO()):
            runner.run([nasl_file])

        compare_content = (
            "\tIncluded Plugins: CheckMissingDescExit\n\t"
            "Running plugins: check_missing_desc_exit\n\n\nChecking"
            "Run plugin check_duplicate_oid"
            f"\t\t{get_path_from_root(nasl_file, self.root)}: "
            "Invalid OID 1.2.3.4.5.6.78909.1.7.654321 found."
            f" {get_path_from_root(nasl_file, self.root)} (0/1)\n"
            "\tNo results for plugin"
            " check_missing_desc_exit\n\tTime elapsed: 0:00:00.013967"
        )
        gen_content = gen_log_file.read_text(encoding="utf-8")
        gen_log_file.unlink()
        # Remove Time elapsed line
        self.assertNotEqual(
            compare_content.splitlines()[:-1],
            gen_content.splitlines()[:-1],
        )

    def test_runner_run_ok_with_ignore_warnings(self):
        included_plugins = [
            "CheckCVEFormat",
        ]
        nasl_file = _here / "plugins" / "test_files" / "nasl" / "warning.nasl"
        content = nasl_file.read_text(encoding=CURRENT_ENCODING)

        reporter = Reporter(term=self._term, root=self.root)

        runner = Runner(
            n_jobs=1,
            reporter=reporter,
            included_plugins=included_plugins,
            root=self.root,
            ignore_warnings=True,
        )

        with redirect_stdout(io.StringIO()) as f:
            runner.run([nasl_file])

            new_content = nasl_file.read_text(encoding=CURRENT_ENCODING)
            self.assertEqual(content, new_content)

        output = f.getvalue()
        self.assertIn(f"{'sum':48} {0:8} {0:8}", output)

    def test_runner_run_fail_without_ignore_warnings(self):
        included_plugins = [
            "CheckCVEFormat",
        ]
        nasl_file = _here / "plugins" / "test_files" / "nasl" / "warning.nasl"
        content = nasl_file.read_text(encoding=CURRENT_ENCODING)

        reporter = Reporter(term=self._term, root=self.root)

        runner = Runner(
            n_jobs=1,
            reporter=reporter,
            included_plugins=included_plugins,
            root=self.root,
        )

        with redirect_stdout(io.StringIO()) as f:
            runner.run([nasl_file])

            new_content = nasl_file.read_text(encoding=CURRENT_ENCODING)
            self.assertEqual(content, new_content)

        output = f.getvalue()
        self.assertIn(f"{'sum':48} {0:8} {1:8}", output)

    def test_runner_log_file_statistic(self):
        included_plugins = [
            CheckDuplicateOID.name,
            CheckMissingDescExit.name,
            CheckNoSolution.name,
        ]
        nasl_file = (
            _here
            / "plugins"
            / "test_files"
            / "nasl"
            / "21.04"
            / "runner"
            / "test.nasl"
        )
        gen_log_file = _here / "gen_log.txt"

        reporter = Reporter(
            term=self._term,
            root=self.root,
            verbose=3,
            log_file_statistic=gen_log_file,
            ignore_warnings=True,
        )

        runner = Runner(
            reporter=reporter,
            n_jobs=1,
            included_plugins=included_plugins,
            root=self.root,
        )
        with redirect_stdout(io.StringIO()):
            runner.run([nasl_file])

        compare_content = (
            f"{'Plugin':50} Errors\n"
            f"{'-' * 59}\n"
            f"{'check_duplicate_oid':48} {1:8}\n"
            f"{'-' * 59}\n"
            f"{'sum':48} {1:8}\n"
        )
        gen_content = gen_log_file.read_text(encoding="utf-8")
        gen_log_file.unlink()

        self.assertEqual(compare_content, gen_content)

    def test_runner_fail_log_file_statistic(self):
        included_plugins = [
            CheckDuplicateOID.name,
            CheckMissingDescExit.name,
            CheckNoSolution.name,
        ]
        nasl_file = (
            _here
            / "plugins"
            / "test_files"
            / "nasl"
            / "21.04"
            / "runner"
            / "test.nasl"
        )
        gen_log_file = _here / "gen_log.txt"

        reporter = Reporter(
            term=self._term,
            root=self.root,
            verbose=3,
            log_file_statistic=gen_log_file,
            ignore_warnings=True,
            fix=True,
        )

        runner = Runner(
            reporter=reporter,
            n_jobs=1,
            included_plugins=included_plugins,
            root=self.root,
        )
        with redirect_stdout(io.StringIO()):
            runner.run([nasl_file])

        compare_content = (
            f"{'Plugin':50} Errors\n"
            f"{'-' * 59}\n"
            f"{'check_duplicate_oid':48} {1:8}\n"
            f"{'-' * 59}\n"
            f"{'sum':48} {1:8}\n"
        )
        gen_content = gen_log_file.read_text(encoding="utf-8")
        gen_log_file.unlink()

        self.assertNotEqual(compare_content, gen_content)



#  Copyright (c) 2022 Greenbone Networks GmbH
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from pathlib import Path
from typing import Iterator

from troubadix.plugin import FileContentPlugin, LinterError, LinterResult


class CheckMissingDescExit(FileContentPlugin):
    name = "check_missing_desc_exit"

    def check_content(
        self,
        nasl_file: Path,
        file_content: str,
    ) -> Iterator[LinterResult]:
        """This script checks if a VT is missing an 'exit(0);' within the
        description block like this:

        if(description) {
          *tags*
        }

        which should be the following instead:

        if(description) {
          *tags*
          exit(0);
        }

        Args:
            nasl_file: The VT that is going to be checked
            file_content: The content of the file that is going to be checked

        """
        if nasl_file.suffix == ".inc":
            return

        match = re.search(
            r"^if\s*\(\s*description\s*\)\s*\{(.+?)^\}",
            file_content,
            re.MULTILINE | re.DOTALL,
        )
        if match and match.group(1):
            submatch = re.search(
                r"^\s*exit\s*\(\s*0\s*\)\s*;", match.group(1), re.MULTILINE
            )
            if not submatch:
                yield LinterError(
                    "No mandatory exit(0); found in the description block.",
                    file=nasl_file,
                    plugin=self.name,
                )

            return

        yield LinterError(
            "No description block extracted/found.",
            file=nasl_file,
            plugin=self.name,
        )



#  Copyright (c) 2022 Greenbone Networks GmbH
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from typing import Iterator

from troubadix.helper import (
    CURRENT_ENCODING,
    SpecialScriptTag,
    get_path_from_root,
    get_special_script_tag_pattern,
)
from troubadix.plugin import FilesPlugin, LinterError, LinterResult

# import json

OPENVAS_OID_PREFIX = r"1.3.6.1.4.1.25623.1.[0-9]+."
OID_RE = re.compile(r"^1\.3\.6\.1\.4\.1\.25623\.1\.[0-9]+\.[\d.]+$")

KNOWN_DUPS = {"1.3.6.1.4.1.25623.1.0.850001", "1.3.6.1.4.1.25623.1.0.95888"}
KNOWN_ABSENTS = {"template.nasl"}


class CheckDuplicateOID(FilesPlugin):
    name = "check_duplicate_oid"

    def run(self) -> Iterator[LinterResult]:
        """Run PRE_RUN_COLLECTOR."""

        mapping = dict()

        for nasl_file in self.context.nasl_files:
            if not nasl_file.suffix == ".nasl":
                continue

            nasl_file_root = get_path_from_root(nasl_file, self.context.root)

            oid = None
            content = nasl_file.read_text(encoding=CURRENT_ENCODING)
            match = get_special_script_tag_pattern(SpecialScriptTag.OID).search(
                content
            )

            if match:
                oid = match.group("oid")

            if not oid:
                yield LinterError(
                    f"Could not find an OID in '{nasl_file_root}'.",
                    plugin=self.name,
                    file=nasl_file,
                )

            elif not OID_RE.match(oid):
                yield LinterError(
                    f"Invalid OID {oid} found in '{nasl_file_root}'.",
                    plugin=self.name,
                    file=nasl_file,
                )

            elif oid not in mapping:
                mapping[oid] = nasl_file_root
            else:
                yield LinterError(
                    f"OID {oid} already used by '{mapping[oid]}'",
                    file=nasl_file,
                    plugin=self.name,
                )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import unittest
from pathlib import Path
from unittest.mock import Mock

from pontos.terminal import Terminal

from troubadix.troubadix import generate_file_list, generate_patterns


class TestNASLinter(unittest.TestCase):
    def setUp(self):
        # store old arguments
        self.old_args = sys.argv

    def tearDown(self) -> None:
        # reset old arguments
        sys.argv = self.old_args

    def test_generate_file_list_with_exclude_patterns(self):
        cwd = Path.cwd()
        files = generate_file_list(
            dirs=[cwd],
            exclude_patterns=[
                "**/test.nasl",
                "**/*.inc",
                "**/templates/*/*.nasl",
                "**/test_files/*",
                "**/test_files/**/*.nasl",
            ],
            include_patterns=["**/*.nasl", "**/*.inc"],
        )
        expected_files = [
            Path(f"{cwd}/tests/plugins/fail.nasl"),
            Path(f"{cwd}/tests/plugins/fail2.nasl"),
        ]
        expected_files.sort()
        files.sort()

        self.assertEqual(files, expected_files)

    def test_generate_file_list_with_include_patterns(self):
        cwd = Path.cwd()
        files = generate_file_list(
            dirs=[cwd],
            exclude_patterns=None,
            include_patterns=["**/tests/*/*.nasl", "**/tests/*/*.inc"],
        )
        expected_files = [
            Path(f"{cwd}/tests/plugins/fail.nasl"),
            Path(f"{cwd}/tests/plugins/test.nasl"),
            Path(f"{cwd}/tests/plugins/fail2.nasl"),
        ]
        expected_files.sort()
        files.sort()

        self.assertEqual(files, expected_files)

    def test_generate_patterns_non_recursive(self):
        terminal = Mock(spec=Terminal)
        include_patterns = ["*.nasl", "*.inc"]
        exclude_patterns = ["test.nasl", "templates/*/*.nasl"]

        new_include_patterns, new_exclude_patterns = generate_patterns(
            terminal=terminal,
            include_patterns=include_patterns,
            exclude_patterns=exclude_patterns,
            non_recursive=True,
        )

        self.assertEqual(new_include_patterns, include_patterns)
        self.assertEqual(new_exclude_patterns, exclude_patterns)

    def test_generate_patterns_recursive(self):
        terminal = Mock(spec=Terminal)
        include_patterns = ["*.nasl", "*.inc"]
        exclude_patterns = ["test.nasl", "templates/*/*.nasl"]

        new_include_patterns, new_exclude_patterns = generate_patterns(
            terminal=terminal,
            include_patterns=include_patterns,
            exclude_patterns=exclude_patterns,
            non_recursive=False,
        )

        expected_include_patterns = ["**/*.nasl", "**/*.inc"]
        expected_exclude_patterns = ["**/test.nasl", "**/templates/*/*.nasl"]

        self.assertEqual(new_include_patterns, expected_include_patterns)
        self.assertEqual(new_exclude_patterns, expected_exclude_patterns)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tests',
 'tests.helper',
 'tests.plugins',
 'tests.standalone_plugins',
 'troubadix',
 'troubadix.helper',
 'troubadix.plugins',
 'troubadix.standalone_plugins']

package_data = \
{'': ['*'],
 'tests.plugins': ['test_files/*',
                   'test_files/nasl/*',
                   'test_files/nasl/21.04/*',
                   'test_files/nasl/21.04/runner/*'],
 'troubadix': ['codespell/*']}

install_requires = \
['chardet>=4,<6',
 'codespell>=2.0.0,<3.0.0',
 'pontos>=22.7.0,<23.0.0',
 'python-magic>=0.4.25,<0.5.0',
 'validators>=0.18.2,<0.21.0']

entry_points = \
{'console_scripts': ['troubadix = troubadix.troubadix:main',
                     'troubadix-changed-oid = '
                     'troubadix.standalone_plugins.changed_oid:main',
                     'troubadix-last-modification = '
                     'troubadix.standalone_plugins.last_modification:main',
                     'troubadix-version-updated = '
                     'troubadix.standalone_plugins.version_updated:main']}

setup_kwargs = {
    'name': 'troubadix',
    'version': '22.9.3',
    'description': 'A linting and QA check tool for NASL files',
    'long_description': '![Greenbone Logo](https://www.greenbone.net/wp-content/uploads/gb_new-logo_horizontal_rgb_small.png)\n\n# Troubadix\nA linting and QA check tool for NASL files\n\n[![GitHub releases](https://img.shields.io/github/release/greenbone/troubadix.svg)](https://github.com/greenbone/troubadix/releases)\n[![PyPI release](https://img.shields.io/pypi/v/troubadix.svg)](https://pypi.org/project/troubadix/)\n[![codecov](https://codecov.io/gh/greenbone/troubadix/branch/main/graph/badge.svg?token=FFMmVmAmtb)](https://codecov.io/gh/greenbone/troubadix)\n[![Build and test](https://github.com/greenbone/troubadix/actions/workflows/ci-python.yml/badge.svg)](https://github.com/greenbone/troubadix/actions/workflows/ci-python.yml)\n\n\n## Installation\n\n### Requirements\n\nPython 3.7 and later is supported.\n\n### Install using pip\n\npip 19.0 or later is required.\n\nYou can install the latest stable release of **troubadix** from the Python\nPackage Index (pypi) using [pip]\n\n    python3 -m pip install --user troubadix\n\n### Install using poetry\n\nBecause **troubadix** is a Python application you most likely need a tool to\nhandle Python package dependencies and Python environments. Therefore we\nstrongly recommend using [pipenv] or [poetry].\n\nYou can install the latest stable release of **troubadix** and add it as\na dependency for your current project using [poetry]\n\n    poetry add troubadix\n\nFor installation via pipenv please take a look at their [documentation][pipenv].\n\n## Development\n\n**troubadix** uses [poetry] for its own dependency management and build\nprocess.\n\nFirst install poetry via pip\n\n    python3 -m pip install --user poetry\n\nAfterwards run\n\n    poetry install\n\nin the checkout directory of **troubadix** (the directory containing the\n`pyproject.toml` file) to install all dependencies including the packages only\nrequired for development.\n\nAfterwards activate the git hooks for auto-formatting and linting via\n[autohooks].\n\n    poetry run autohooks activate\n\nValidate the activated git hooks by running\n\n    poetry run autohooks check\n\n## Maintainer\n\nThis project is maintained by [Greenbone Networks GmbH][Greenbone Networks]\n\n## Contributing\n\nYour contributions are highly appreciated. Please\n[create a pull request](https://github.com/greenbone/troubadix/pulls)\non GitHub. Bigger changes need to be discussed with the development team via the\n[issues section at GitHub](https://github.com/greenbone/troubadix/issues)\nfirst.\n\n## License\n\nCopyright (C) 2021-2022 [Greenbone Networks GmbH][Greenbone Networks]\n\nLicensed under the [GNU General Public License v3.0 or later](LICENSE).\n\n[Greenbone Networks]: https://www.greenbone.net/\n[poetry]: https://python-poetry.org/\n[pip]: https://pip.pypa.io/\n[pipenv]: https://pipenv.pypa.io/\n[autohooks]: https://github.com/greenbone/autohooks\n',
    'author': 'Greenbone',
    'author_email': 'info@greenbone.net',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/greenbone/troubadix',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



#  Copyright (c) 2022 Greenbone Networks GmbH
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

import io
import re
from contextlib import redirect_stdout
from pathlib import Path
from typing import Iterator

from codespell_lib import main as codespell_main

from troubadix.plugin import FilesPlugin, LinterError, LinterResult

plugin_path = Path(__file__).parent.resolve()
codespell_config_path = (plugin_path.parent / "codespell").resolve()


class CheckSpelling(FilesPlugin):
    name = "check_spelling"

    def run(self) -> Iterator[LinterResult]:
        """This script checks, via the codespell library, wether
        the provided nasl files contain spelling errors.
        Certain errors are ignored based on listed exceptions

        Yields:
            Iterator[LinterResult]: The detected spelling errors
        """

        # Overwrite with local repository files if exist
        if Path("codespell.additions").exists():
            codespell_additions = Path("codespell.additions").resolve()
        else:
            codespell_additions = f"{codespell_config_path}/codespell.additions"
        if Path("codespell.exclude").exists():
            codespell_exclude = Path("codespell.exclude").resolve()
        else:
            codespell_exclude = f"{codespell_config_path}/codespell.exclude"
        if Path("codespell.ignore").exists():
            codespell_ignore = Path("codespell.ignore").resolve()
        else:
            codespell_ignore = f"{codespell_config_path}/codespell.ignore"

        batch_size = 10_000
        for i in range(0, len(self.context.nasl_files), batch_size):

            files_parameters = [
                str(nasl_file)
                for nasl_file in self.context.nasl_files[i : i + batch_size]
            ]
            codespell_arguments = [
                "--hard-encoding-detection",
                "--dictionary=-",
                f"--dictionary={codespell_additions}",
                f"--exclude-file={codespell_exclude}",
                f"--ignore-words={codespell_ignore}",
                "--disable-colors",
            ] + files_parameters

            with redirect_stdout(io.StringIO()) as codespell:
                codespell_main(*codespell_arguments)

            codespell = codespell.getvalue()
            if "Traceback (most recent call last):" not in codespell:
                _codespell = codespell.splitlines()
                codespell = ""
                for line in _codespell:

                    # From /Policy which is just a huge blob of text
                    # and too large for codespell.exclude:
                    if "policy_file_checksums_win.nasl" in line:
                        if re.search(r"nD\s+==>\s+and, 2nd", line) or re.search(
                            r"oD\s+==>\s+of", line
                        ):
                            continue

                    # Same for a few other files:
                    if "smtp_AV_42zip_DoS.nasl" in line and re.search(
                        r"BA\s+==>\s+BY, BE", line
                    ):
                        continue

                    if "bad_ssh_host_keys.inc" in line and re.search(
                        r"ba\s+==>\s+by, be", line
                    ):
                        continue

                    if "wmi_misc.inc" in line:
                        if re.search(r"BA\s+==>\s+BY, BE", line) or re.search(
                            r"OD\s+==>\s+OF", line
                        ):
                            continue

                    if (
                        "ssl_funcs.inc" in line
                        or "gb_ssl_tls_cert_details.nasl" in line
                    ):
                        if re.search(r"fpr\s+==>\s+for, far, fps", line):
                            continue

                    # Codespell has currently cna->can in the dictionary.txt
                    # which is causing false positives for CNA (widely used term
                    # in VTs) because codespell doesn't look at the casing. For
                    # now we're excluding any uppercase "CNA" results because
                    # these are usually false positives we don't want to report.
                    # Similar with "RO" because that is also very likely a false
                    # positive.
                    if re.search(r"CNA\s+==>\s+CAN", line) or re.search(
                        r"RO\s+==>\s+TO", line
                    ):
                        continue

                    # Name of a Huawei product
                    if (
                        "gb_huawei" in line
                        or "telnetserver_detect_type_nd_version.nasl" in line
                    ):
                        if re.search(
                            r"eSpace\s+==>\s+escape", line, flags=re.IGNORECASE
                        ):
                            continue

                    # "ure" is a Debian package, again too many hits for
                    # codespell.exclude.
                    if re.search(
                        r"(deb_(dla_)?[0-9]+(_[" r"0-9]+)?|gb_ubuntu_.+)\.nasl",
                        line,
                    ):
                        if re.search(r"ure\s+==>\s+sure", line):
                            continue

                    # gsf/PCIDSS VTs are currently using some german text parts
                    # nb: codespell seems to have some issues with
                    # german umlauts in the codespell.exclude so a few of these
                    # were also excluded here instead of directly
                    # via codespell.exclude.
                    if (
                        "PCIDSS/" in line
                        or "GSHB/" in line
                        or "attic/PCIDSS_" in line
                        or "ITG_Kompendium/" in line
                    ):
                        if re.search(
                            r"(sie|ist|oder|prozess|manuell|unter|funktion|"
                            r"alle|als|tage|lokale|uptodate|paket|titel|ba|"
                            r"ordner|modul|interaktive|programm|explizit|"
                            r"normale|applikation|attributen|lokal|signatur|"
                            r"modell|klick|generell)\s+==>\s+",
                            line,
                            flags=re.IGNORECASE,
                        ):
                            continue

                    # False positives in the gsf/PCIDSS and GSHB/ VTs:
                    # string('\nIn the file sent\nin milliseconds
                    # There are too many hits to maintain
                    # them in codespell.exclude so exclude them for now here.
                    if (
                        "PCIDSS/" in line
                        or "GSHB/" in line
                        or "attic/PCIDSS_" in line
                    ):
                        if re.search(r"n[iI]n\s+==>\s+inn", line):
                            continue

                    # False positives in the GSHB/ and ITG_Kompendium/ VTs on
                    # bsi.bund.de URLs.
                    if (
                        "GSHB/" in line
                        or "ITG_Kompendium/" in line
                        or "Policy/gb_policy_cipher_suites.nasl" in line
                        or "Policy/policy_BSI-TR-03116-4.nasl" in line
                        or "2012/gb_secpod_ssl_ciphers_weak_report.nasl" in line
                    ):
                        if re.search(r"bund\s+==>\s+bind", line):
                            continue

                    # False positive in this VT in German example responses.
                    if "gb_exchange_server_CVE-2021-26855_active.nasl" in line:
                        if re.search(r"ist\s+==>\s+is", line):
                            continue

                    # Mostly a false positive in LSCs because of things like
                    # "ALSA: hda" or a codec called "Conexant". There are too
                    # many hits to maintain them in codespell.exclude so exclude
                    # them for now here.
                    if re.search(
                        r"gb_(sles|(open)?suse|ubuntu_USN)_.+\.nasl", line
                    ):
                        if re.search(
                            r"(hda|conexant)\s+==>\s+(had|connexant)",
                            line,
                            flags=re.IGNORECASE,
                        ):
                            continue

                    # Jodie Chancel is a security researcher who is mentioned
                    # many times in Mozilla advisories
                    if re.search(
                        r"gb_mozilla_firefox_mfsa_\d{4}-\d{2,4}_lin\.nasl", line
                    ) and re.search(r"Chancel\s+==>\s+Cancel", line):
                        continue

                    # Look like correct as this is also in dictionary_rare.txt
                    if "deb_dla_2896.nasl" in line and re.search(
                        r"dependant\s+==>\s+dependent", line
                    ):
                        continue

                    # Similar to the one above for e.g. SLES.
                    # Also exclude "tre", because it's a package name.
                    if re.search(r"mgasa-\d{4}-\d{4}.nasl", line) and re.search(
                        r"(hda|tre|conexant)\s+==>\s+(had|tree|connexant)",
                        line,
                        flags=re.IGNORECASE,
                    ):
                        continue

                    # Similar to the corrections above, with some additional
                    # exclusions like e.g. names
                    if re.search(r"ELSA-\d{4}-\d{4,5}\.nasl", line):
                        if (
                            re.search(r"Stange\s+==>\s+Strange", line)
                            or re.search(r"chang\s+==>\s+change, charge", line)
                            or re.search(
                                r"IST\s+==>\s+IS, IT, ITS, IT'S, SIT, LIST",
                                line,
                            )
                            or re.search(r"hda\s+==>\s+had", line)
                            or re.search(r"Readded\s+==>\s+Read", line)
                            or re.search(
                                r"ACI\s+==>\s+ACPI", line, re.IGNORECASE
                            )
                            or re.search(r"UE\s+==>\s+USE, DUE", line)
                        ):
                            continue

                    # "Unsecure" is used in the references so we shouldn't
                    # change that.
                    if (
                        "office2013_allow_insecure_apps_catalogs.nasl" in line
                        or "gb_sap_rfc_default_pw.nasl" in line
                        or "gb_sap_webgui_default_pw.nasl" in line
                    ):
                        if re.search(r"[Uu]nsecure\s+==>\s+[Ii]nsecure", line):
                            continue

                    codespell += line + "\n"

            for codespell_entry in codespell.splitlines():
                if "==>" in codespell:
                    yield LinterError(
                        codespell_entry,
                        file=codespell_entry.split(":")[0],
                        plugin=self.name,
                    )
                elif "Traceback (most recent call last):" in codespell:
                    yield LinterError(
                        codespell,
                        plugin=self.name,
                    )



# Copyright (C) 2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path

from troubadix.plugins.copyright_year import CheckCopyrightYear

from . import PluginTestCase


class CheckCopyrightYearTestCase(PluginTestCase):
    def test_ok(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"creation_date", value:"2022-05-14 11:24:55 '
            '+0200 (Tue, 14 May 2022)");',
            'script_copyright("Copyright (C) 2022 Greenbone Networks GmbH");',
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, lines=content
        )
        plugin = CheckCopyrightYear(fake_context)

        results = list(plugin.run())
        self.assertEqual(len(results), 0)

    def test_exclude_inc_file(self):
        path = Path("some/file.inc")
        fake_context = self.create_file_plugin_context(nasl_file=path)
        plugin = CheckCopyrightYear(fake_context)

        results = list(plugin.run())

        self.assertEqual(len(results), 0)

    def test_pre_ok(self):
        path = Path("some/pre2008/file.nasl")
        content = (
            'script_tag(name:"creation_date", value:"2022-05-14 11:24:55 '
            '+0200 (Tue, 14 May 2022)");',
            'script_copyright("Copyright (C) 2020 Greenbone Networks GmbH");',
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, lines=content
        )
        plugin = CheckCopyrightYear(fake_context)

        results = list(plugin.run())
        self.assertEqual(len(results), 0)

    def test_pre_fail(self):
        path = Path("some/pre2008/file.nasl")
        content = (
            'script_tag(name:"creation_date", value:"2020-05-14 11:24:55 '
            '+0200 (Tue, 14 May 2020)");',
            'script_copyright("Copyright (C) 2021 Greenbone Networks GmbH");',
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, lines=content
        )
        plugin = CheckCopyrightYear(fake_context)

        results = list(plugin.run())
        self.assertEqual(len(results), 1)
        self.assertEqual(
            "VT contains a Copyright year not matching "
            "the year 2020 at line 2",
            results[0].message,
        )

    def test_missing_creation_date(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"qod_type", value:"remote_banner");',
            'script_family("Product detection");',
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, lines=content
        )
        plugin = CheckCopyrightYear(fake_context)

        results = list(plugin.run())
        self.assertEqual(len(results), 1)
        self.assertEqual(
            "Missing creation_date statement in VT", results[0].message
        )

    def test_creation_date_not_copyright_year(self):
        path = Path("some/file.nasl")
        content = (
            'script_tag(name:"creation_date", value:"2022-05-14 11:24:55 '
            '+0200 (Tue, 14 May 2013)");',
            'script_copyright("Copyright (C) 2020 Greenbone Networks GmbH");',
        )
        fake_context = self.create_file_plugin_context(
            nasl_file=path, lines=content
        )
        plugin = CheckCopyrightYear(fake_context)

        results = list(plugin.run())
        self.assertEqual(len(results), 1)
        self.assertEqual(
            "VT contains a Copyright year not matching "
            "the year 2022 at line 2",
            results[0].message,
        )
